# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.driver_efficiency_get_driver_efficiency_by_drivers_response_body import (
    DriverEfficiencyGetDriverEfficiencyByDriversResponseBody,
)
from ..types.driver_efficiency_get_driver_efficiency_by_vehicles_response_body import (
    DriverEfficiencyGetDriverEfficiencyByVehiclesResponseBody,
)
from ..types.fuel_energy_get_fuel_energy_driver_reports_response_body import (
    FuelEnergyGetFuelEnergyDriverReportsResponseBody,
)
from ..types.fuel_energy_get_fuel_energy_vehicle_reports_response_body import (
    FuelEnergyGetFuelEnergyVehicleReportsResponseBody,
)
from ..types.fuel_purchase_post_fuel_purchase_response_body import FuelPurchasePostFuelPurchaseResponseBody
from ..types.post_fuel_purchase_request_body_discount_request_body import PostFuelPurchaseRequestBodyDiscountRequestBody
from ..types.post_fuel_purchase_request_body_price_request_body import PostFuelPurchaseRequestBodyPriceRequestBody
from .raw_client import AsyncRawFuelAndEnergyClient, RawFuelAndEnergyClient
from .types.fuel_purchase_post_fuel_purchase_request_body_fuel_grade import (
    FuelPurchasePostFuelPurchaseRequestBodyFuelGrade,
)
from .types.fuel_purchase_post_fuel_purchase_request_body_ifta_fuel_type import (
    FuelPurchasePostFuelPurchaseRequestBodyIftaFuelType,
)
from .types.get_fuel_energy_vehicle_reports_request_energy_type import GetFuelEnergyVehicleReportsRequestEnergyType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class FuelAndEnergyClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawFuelAndEnergyClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawFuelAndEnergyClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawFuelAndEnergyClient
        """
        return self._raw_client

    def get_driver_efficiency_by_drivers(
        self,
        *,
        start_time: str,
        end_time: str,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        data_formats: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverEfficiencyGetDriverEfficiencyByDriversResponseBody:
        """
        This endpoint will return driver efficiency data that has been collected for your organization and grouped by drivers based on the time parameters passed in. Results are paginated.

        **Note:** The data from this endpoint comes from the Driver Efficiency (Eco-Driving) Report. The existing [/fleet/drivers/efficiency](https://developers.samsara.com/reference/getdriverefficiency) endpoint has now been moved to Legacy.

         <b>Rate limit:</b> 10 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Driver Efficiency** under the Fuel & Energy category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Must be in multiple of hours and at least 1 day before endTime. Timezones are supported. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. (Examples: 2019-06-11T19:00:00Z, 2015-09-12T14:00:00-04:00).

        end_time : str
            An end time in RFC 3339 format. Must be in multiple of hours and no later than 3 hours before the current time. Timezones are supported. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. (Examples: 2019-06-13T19:00:00Z, 2015-09-15T14:00:00-04:00).

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        data_formats : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list of data formats you want to fetch. Valid values: `score`, `raw` and `percentage`. The default data format is `score`. Example: `dataFormats=raw,score`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverEfficiencyGetDriverEfficiencyByDriversResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.fuel_and_energy.get_driver_efficiency_by_drivers(
            start_time="startTime",
            end_time="endTime",
        )
        """
        _response = self._raw_client.get_driver_efficiency_by_drivers(
            start_time=start_time,
            end_time=end_time,
            driver_ids=driver_ids,
            data_formats=data_formats,
            tag_ids=tag_ids,
            parent_tag_ids=parent_tag_ids,
            after=after,
            request_options=request_options,
        )
        return _response.data

    def get_driver_efficiency_by_vehicles(
        self,
        *,
        start_time: str,
        end_time: str,
        vehicle_ids: typing.Optional[str] = None,
        data_formats: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverEfficiencyGetDriverEfficiencyByVehiclesResponseBody:
        """
        This endpoint will return driver efficiency data that has been collected for your organization and grouped by vehicle drivers used based on the time parameters passed in. Results are paginated.

        **Note:** The data from this endpoint comes from the Driver Efficiency (Eco-Driving) Report. The existing [/fleet/drivers/efficiency](https://developers.samsara.com/reference/getdriverefficiency) endpoint has now been moved to Legacy.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Driver Efficiency** under the Fuel & Energy category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Must be in multiple of hours and at least 1 day before endTime. Timezones are supported. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. (Examples: 2019-06-11T19:00:00Z, 2015-09-12T14:00:00-04:00).

        end_time : str
            An end time in RFC 3339 format. Must be in multiple of hours and no later than 3 hours before the current time. Timezones are supported. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. (Examples: 2019-06-13T19:00:00Z, 2015-09-15T14:00:00-04:00).

        vehicle_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle IDs and externalIds. Example: `vehicleIds=1234,5678,samsara.vin:1HGBH41JXMN109186`

        data_formats : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list of data formats you want to fetch. Valid values: `score`, `raw` and `percentage`. The default data format is `score`. Example: `dataFormats=raw,score`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverEfficiencyGetDriverEfficiencyByVehiclesResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.fuel_and_energy.get_driver_efficiency_by_vehicles(
            start_time="startTime",
            end_time="endTime",
        )
        """
        _response = self._raw_client.get_driver_efficiency_by_vehicles(
            start_time=start_time,
            end_time=end_time,
            vehicle_ids=vehicle_ids,
            data_formats=data_formats,
            tag_ids=tag_ids,
            parent_tag_ids=parent_tag_ids,
            after=after,
            request_options=request_options,
        )
        return _response.data

    def get_fuel_energy_driver_reports(
        self,
        *,
        start_date: str,
        end_date: str,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FuelEnergyGetFuelEnergyDriverReportsResponseBody:
        """
        Get fuel and energy efficiency driver reports for the requested time range.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Fuel & Energy** under the Fuel & Energy category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_date : str
            A start date in RFC 3339 format. This parameter ignores everything (i.e. hour, minutes, seconds, nanoseconds, etc.) besides the date and timezone. If no time zone is passed in, then the UTC time zone will be used. This parameter is inclusive, so data on the date specified will be considered. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. For example, 2022-07-13T14:20:50.52-07:00 is a time in Pacific Daylight Time.

        end_date : str
            An end date in RFC 3339 format. This parameter ignores everything (i.e. hour, minutes, seconds, nanoseconds, etc.) besides the date and timezone. If no time zone is passed in, then the UTC time zone will be used. This parameter is inclusive, so data on the date specified will be considered. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. For example, 2022-07-13T14:20:50.52-07:00 is a time in Pacific Daylight Time.

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FuelEnergyGetFuelEnergyDriverReportsResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.fuel_and_energy.get_fuel_energy_driver_reports(
            start_date="startDate",
            end_date="endDate",
        )
        """
        _response = self._raw_client.get_fuel_energy_driver_reports(
            start_date=start_date,
            end_date=end_date,
            driver_ids=driver_ids,
            tag_ids=tag_ids,
            parent_tag_ids=parent_tag_ids,
            after=after,
            request_options=request_options,
        )
        return _response.data

    def get_fuel_energy_vehicle_reports(
        self,
        *,
        start_date: str,
        end_date: str,
        vehicle_ids: typing.Optional[str] = None,
        energy_type: typing.Optional[GetFuelEnergyVehicleReportsRequestEnergyType] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FuelEnergyGetFuelEnergyVehicleReportsResponseBody:
        """
        Get fuel and energy efficiency vehicle reports for the requested time range.

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Fuel & Energy** under the Fuel & Energy category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_date : str
            A start date in RFC 3339 format. This parameter ignores everything (i.e. hour, minutes, seconds, nanoseconds, etc.) besides the date and timezone. If no time zone is passed in, then the UTC time zone will be used. This parameter is inclusive, so data on the date specified will be considered. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. For example, 2022-07-13T14:20:50.52-07:00 is a time in Pacific Daylight Time.

        end_date : str
            An end date in RFC 3339 format. This parameter ignores everything (i.e. hour, minutes, seconds, nanoseconds, etc.) besides the date and timezone. If no time zone is passed in, then the UTC time zone will be used. This parameter is inclusive, so data on the date specified will be considered. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. For example, 2022-07-13T14:20:50.52-07:00 is a time in Pacific Daylight Time.

        vehicle_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle IDs and externalIds. Example: `vehicleIds=1234,5678,samsara.vin:1HGBH41JXMN109186`

        energy_type : typing.Optional[GetFuelEnergyVehicleReportsRequestEnergyType]
            The type of energy used by the vehicle.  Valid values: `fuel`, `hybrid`, `electric`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FuelEnergyGetFuelEnergyVehicleReportsResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.fuel_and_energy.get_fuel_energy_vehicle_reports(
            start_date="startDate",
            end_date="endDate",
        )
        """
        _response = self._raw_client.get_fuel_energy_vehicle_reports(
            start_date=start_date,
            end_date=end_date,
            vehicle_ids=vehicle_ids,
            energy_type=energy_type,
            tag_ids=tag_ids,
            parent_tag_ids=parent_tag_ids,
            after=after,
            request_options=request_options,
        )
        return _response.data

    def post_fuel_purchase(
        self,
        *,
        fuel_quantity_liters: str,
        transaction_location: str,
        transaction_price: PostFuelPurchaseRequestBodyPriceRequestBody,
        transaction_reference: str,
        transaction_time: str,
        discount: typing.Optional[PostFuelPurchaseRequestBodyDiscountRequestBody] = OMIT,
        driver_id: typing.Optional[str] = OMIT,
        fuel_grade: typing.Optional[FuelPurchasePostFuelPurchaseRequestBodyFuelGrade] = OMIT,
        ifta_fuel_type: typing.Optional[FuelPurchasePostFuelPurchaseRequestBodyIftaFuelType] = OMIT,
        merchant_name: typing.Optional[str] = OMIT,
        source: typing.Optional[str] = OMIT,
        vehicle_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FuelPurchasePostFuelPurchaseResponseBody:
        """
        Create a fuel purchase transaction.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Fuel Purchase** under the Fuel & Energy category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        fuel_quantity_liters : str
            The amount of fuel purchased in liters.

        transaction_location : str
            The full street address for the location of the fuel transaction, as it might be recognized by Google Maps. Ideal entries should be in accordance with the format used by the national postal service of the country concerned (example: 1 De Haro St, San Francisco, CA 94107, United States). Alternatively, exact latitude/longitude can be provided (example: 40.748441, -73.985664).

        transaction_price : PostFuelPurchaseRequestBodyPriceRequestBody

        transaction_reference : str
            The fuel transaction reference. This is the transaction identifier. For instance, this can be the Serial Number on the invoice.

        transaction_time : str
            The time of the fuel transaction in RFC 3339 format. Timezone must be specified. For example, 2022-07-13T14:20:50.52-07:00 is a time in Pacific Daylight Time.

        discount : typing.Optional[PostFuelPurchaseRequestBodyDiscountRequestBody]

        driver_id : typing.Optional[str]
            Samsara ID of the driver that purchased the fuel.

        fuel_grade : typing.Optional[FuelPurchasePostFuelPurchaseRequestBodyFuelGrade]
            The grade of the fuel purchased.  Valid values: `Unknown`, `Regular`, `Premium`

        ifta_fuel_type : typing.Optional[FuelPurchasePostFuelPurchaseRequestBodyIftaFuelType]
            The type of fuel purchased supported by IFTA.  Valid values: `Unspecified`, `A55`, `Biodiesel`, `CompressedNaturalGas`, `Diesel`, `E85`, `Electricity`, `Ethanol`, `Gasohol`, `Gasoline`, `Hydrogen`, `LiquifiedNaturalGas`, `M85`, `Methanol`, `Propane`, `Other`

        merchant_name : typing.Optional[str]
            Brand name of the fuel station the fuel was purchased at. For example: Shell, Bp, Exxon.

        source : typing.Optional[str]
            The integration provider. For example: Customer, Shell Integration

        vehicle_id : typing.Optional[str]
            Samsara ID of the vehicle that purchased the fuel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FuelPurchasePostFuelPurchaseResponseBody
            OK response.

        Examples
        --------
        from samsara import PostFuelPurchaseRequestBodyPriceRequestBody, Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.fuel_and_energy.post_fuel_purchase(
            fuel_quantity_liters="676.8",
            transaction_location="350 Rhode Island St, San Francisco, CA 94103",
            transaction_price=PostFuelPurchaseRequestBodyPriceRequestBody(
                amount="640.2",
                currency="usd",
            ),
            transaction_reference="5454534",
            transaction_time="2022-07-13T14:20:50.52-07:00",
        )
        """
        _response = self._raw_client.post_fuel_purchase(
            fuel_quantity_liters=fuel_quantity_liters,
            transaction_location=transaction_location,
            transaction_price=transaction_price,
            transaction_reference=transaction_reference,
            transaction_time=transaction_time,
            discount=discount,
            driver_id=driver_id,
            fuel_grade=fuel_grade,
            ifta_fuel_type=ifta_fuel_type,
            merchant_name=merchant_name,
            source=source,
            vehicle_id=vehicle_id,
            request_options=request_options,
        )
        return _response.data


class AsyncFuelAndEnergyClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawFuelAndEnergyClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawFuelAndEnergyClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawFuelAndEnergyClient
        """
        return self._raw_client

    async def get_driver_efficiency_by_drivers(
        self,
        *,
        start_time: str,
        end_time: str,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        data_formats: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverEfficiencyGetDriverEfficiencyByDriversResponseBody:
        """
        This endpoint will return driver efficiency data that has been collected for your organization and grouped by drivers based on the time parameters passed in. Results are paginated.

        **Note:** The data from this endpoint comes from the Driver Efficiency (Eco-Driving) Report. The existing [/fleet/drivers/efficiency](https://developers.samsara.com/reference/getdriverefficiency) endpoint has now been moved to Legacy.

         <b>Rate limit:</b> 10 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Driver Efficiency** under the Fuel & Energy category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Must be in multiple of hours and at least 1 day before endTime. Timezones are supported. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. (Examples: 2019-06-11T19:00:00Z, 2015-09-12T14:00:00-04:00).

        end_time : str
            An end time in RFC 3339 format. Must be in multiple of hours and no later than 3 hours before the current time. Timezones are supported. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. (Examples: 2019-06-13T19:00:00Z, 2015-09-15T14:00:00-04:00).

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        data_formats : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list of data formats you want to fetch. Valid values: `score`, `raw` and `percentage`. The default data format is `score`. Example: `dataFormats=raw,score`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverEfficiencyGetDriverEfficiencyByDriversResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fuel_and_energy.get_driver_efficiency_by_drivers(
                start_time="startTime",
                end_time="endTime",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_driver_efficiency_by_drivers(
            start_time=start_time,
            end_time=end_time,
            driver_ids=driver_ids,
            data_formats=data_formats,
            tag_ids=tag_ids,
            parent_tag_ids=parent_tag_ids,
            after=after,
            request_options=request_options,
        )
        return _response.data

    async def get_driver_efficiency_by_vehicles(
        self,
        *,
        start_time: str,
        end_time: str,
        vehicle_ids: typing.Optional[str] = None,
        data_formats: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverEfficiencyGetDriverEfficiencyByVehiclesResponseBody:
        """
        This endpoint will return driver efficiency data that has been collected for your organization and grouped by vehicle drivers used based on the time parameters passed in. Results are paginated.

        **Note:** The data from this endpoint comes from the Driver Efficiency (Eco-Driving) Report. The existing [/fleet/drivers/efficiency](https://developers.samsara.com/reference/getdriverefficiency) endpoint has now been moved to Legacy.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Driver Efficiency** under the Fuel & Energy category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Must be in multiple of hours and at least 1 day before endTime. Timezones are supported. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. (Examples: 2019-06-11T19:00:00Z, 2015-09-12T14:00:00-04:00).

        end_time : str
            An end time in RFC 3339 format. Must be in multiple of hours and no later than 3 hours before the current time. Timezones are supported. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. (Examples: 2019-06-13T19:00:00Z, 2015-09-15T14:00:00-04:00).

        vehicle_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle IDs and externalIds. Example: `vehicleIds=1234,5678,samsara.vin:1HGBH41JXMN109186`

        data_formats : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A comma-separated list of data formats you want to fetch. Valid values: `score`, `raw` and `percentage`. The default data format is `score`. Example: `dataFormats=raw,score`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverEfficiencyGetDriverEfficiencyByVehiclesResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fuel_and_energy.get_driver_efficiency_by_vehicles(
                start_time="startTime",
                end_time="endTime",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_driver_efficiency_by_vehicles(
            start_time=start_time,
            end_time=end_time,
            vehicle_ids=vehicle_ids,
            data_formats=data_formats,
            tag_ids=tag_ids,
            parent_tag_ids=parent_tag_ids,
            after=after,
            request_options=request_options,
        )
        return _response.data

    async def get_fuel_energy_driver_reports(
        self,
        *,
        start_date: str,
        end_date: str,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FuelEnergyGetFuelEnergyDriverReportsResponseBody:
        """
        Get fuel and energy efficiency driver reports for the requested time range.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Fuel & Energy** under the Fuel & Energy category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_date : str
            A start date in RFC 3339 format. This parameter ignores everything (i.e. hour, minutes, seconds, nanoseconds, etc.) besides the date and timezone. If no time zone is passed in, then the UTC time zone will be used. This parameter is inclusive, so data on the date specified will be considered. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. For example, 2022-07-13T14:20:50.52-07:00 is a time in Pacific Daylight Time.

        end_date : str
            An end date in RFC 3339 format. This parameter ignores everything (i.e. hour, minutes, seconds, nanoseconds, etc.) besides the date and timezone. If no time zone is passed in, then the UTC time zone will be used. This parameter is inclusive, so data on the date specified will be considered. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. For example, 2022-07-13T14:20:50.52-07:00 is a time in Pacific Daylight Time.

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FuelEnergyGetFuelEnergyDriverReportsResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fuel_and_energy.get_fuel_energy_driver_reports(
                start_date="startDate",
                end_date="endDate",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_fuel_energy_driver_reports(
            start_date=start_date,
            end_date=end_date,
            driver_ids=driver_ids,
            tag_ids=tag_ids,
            parent_tag_ids=parent_tag_ids,
            after=after,
            request_options=request_options,
        )
        return _response.data

    async def get_fuel_energy_vehicle_reports(
        self,
        *,
        start_date: str,
        end_date: str,
        vehicle_ids: typing.Optional[str] = None,
        energy_type: typing.Optional[GetFuelEnergyVehicleReportsRequestEnergyType] = None,
        tag_ids: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FuelEnergyGetFuelEnergyVehicleReportsResponseBody:
        """
        Get fuel and energy efficiency vehicle reports for the requested time range.

         <b>Rate limit:</b> 25 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Fuel & Energy** under the Fuel & Energy category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        start_date : str
            A start date in RFC 3339 format. This parameter ignores everything (i.e. hour, minutes, seconds, nanoseconds, etc.) besides the date and timezone. If no time zone is passed in, then the UTC time zone will be used. This parameter is inclusive, so data on the date specified will be considered. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. For example, 2022-07-13T14:20:50.52-07:00 is a time in Pacific Daylight Time.

        end_date : str
            An end date in RFC 3339 format. This parameter ignores everything (i.e. hour, minutes, seconds, nanoseconds, etc.) besides the date and timezone. If no time zone is passed in, then the UTC time zone will be used. This parameter is inclusive, so data on the date specified will be considered. Note that the most recent 72 hours of data may still be processing and is subject to change and latency, so it is not recommended to request data for the most recent 72 hours. For example, 2022-07-13T14:20:50.52-07:00 is a time in Pacific Daylight Time.

        vehicle_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle IDs and externalIds. Example: `vehicleIds=1234,5678,samsara.vin:1HGBH41JXMN109186`

        energy_type : typing.Optional[GetFuelEnergyVehicleReportsRequestEnergyType]
            The type of energy used by the vehicle.  Valid values: `fuel`, `hybrid`, `electric`

        tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        parent_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FuelEnergyGetFuelEnergyVehicleReportsResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fuel_and_energy.get_fuel_energy_vehicle_reports(
                start_date="startDate",
                end_date="endDate",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_fuel_energy_vehicle_reports(
            start_date=start_date,
            end_date=end_date,
            vehicle_ids=vehicle_ids,
            energy_type=energy_type,
            tag_ids=tag_ids,
            parent_tag_ids=parent_tag_ids,
            after=after,
            request_options=request_options,
        )
        return _response.data

    async def post_fuel_purchase(
        self,
        *,
        fuel_quantity_liters: str,
        transaction_location: str,
        transaction_price: PostFuelPurchaseRequestBodyPriceRequestBody,
        transaction_reference: str,
        transaction_time: str,
        discount: typing.Optional[PostFuelPurchaseRequestBodyDiscountRequestBody] = OMIT,
        driver_id: typing.Optional[str] = OMIT,
        fuel_grade: typing.Optional[FuelPurchasePostFuelPurchaseRequestBodyFuelGrade] = OMIT,
        ifta_fuel_type: typing.Optional[FuelPurchasePostFuelPurchaseRequestBodyIftaFuelType] = OMIT,
        merchant_name: typing.Optional[str] = OMIT,
        source: typing.Optional[str] = OMIT,
        vehicle_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FuelPurchasePostFuelPurchaseResponseBody:
        """
        Create a fuel purchase transaction.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Fuel Purchase** under the Fuel & Energy category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        fuel_quantity_liters : str
            The amount of fuel purchased in liters.

        transaction_location : str
            The full street address for the location of the fuel transaction, as it might be recognized by Google Maps. Ideal entries should be in accordance with the format used by the national postal service of the country concerned (example: 1 De Haro St, San Francisco, CA 94107, United States). Alternatively, exact latitude/longitude can be provided (example: 40.748441, -73.985664).

        transaction_price : PostFuelPurchaseRequestBodyPriceRequestBody

        transaction_reference : str
            The fuel transaction reference. This is the transaction identifier. For instance, this can be the Serial Number on the invoice.

        transaction_time : str
            The time of the fuel transaction in RFC 3339 format. Timezone must be specified. For example, 2022-07-13T14:20:50.52-07:00 is a time in Pacific Daylight Time.

        discount : typing.Optional[PostFuelPurchaseRequestBodyDiscountRequestBody]

        driver_id : typing.Optional[str]
            Samsara ID of the driver that purchased the fuel.

        fuel_grade : typing.Optional[FuelPurchasePostFuelPurchaseRequestBodyFuelGrade]
            The grade of the fuel purchased.  Valid values: `Unknown`, `Regular`, `Premium`

        ifta_fuel_type : typing.Optional[FuelPurchasePostFuelPurchaseRequestBodyIftaFuelType]
            The type of fuel purchased supported by IFTA.  Valid values: `Unspecified`, `A55`, `Biodiesel`, `CompressedNaturalGas`, `Diesel`, `E85`, `Electricity`, `Ethanol`, `Gasohol`, `Gasoline`, `Hydrogen`, `LiquifiedNaturalGas`, `M85`, `Methanol`, `Propane`, `Other`

        merchant_name : typing.Optional[str]
            Brand name of the fuel station the fuel was purchased at. For example: Shell, Bp, Exxon.

        source : typing.Optional[str]
            The integration provider. For example: Customer, Shell Integration

        vehicle_id : typing.Optional[str]
            Samsara ID of the vehicle that purchased the fuel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FuelPurchasePostFuelPurchaseResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara, PostFuelPurchaseRequestBodyPriceRequestBody

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.fuel_and_energy.post_fuel_purchase(
                fuel_quantity_liters="676.8",
                transaction_location="350 Rhode Island St, San Francisco, CA 94103",
                transaction_price=PostFuelPurchaseRequestBodyPriceRequestBody(
                    amount="640.2",
                    currency="usd",
                ),
                transaction_reference="5454534",
                transaction_time="2022-07-13T14:20:50.52-07:00",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.post_fuel_purchase(
            fuel_quantity_liters=fuel_quantity_liters,
            transaction_location=transaction_location,
            transaction_price=transaction_price,
            transaction_reference=transaction_reference,
            transaction_time=transaction_time,
            discount=discount,
            driver_id=driver_id,
            fuel_grade=fuel_grade,
            ifta_fuel_type=ifta_fuel_type,
            merchant_name=merchant_name,
            source=source,
            vehicle_id=vehicle_id,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .create_attribute_request_attribute_type import CreateAttributeRequestAttributeType
    from .create_attribute_request_entity_type import CreateAttributeRequestEntityType
    from .create_attribute_request_unit import CreateAttributeRequestUnit
    from .delete_attributes_request_entity_type import DeleteAttributesRequestEntityType
    from .get_attribute_request_entity_type import GetAttributeRequestEntityType
    from .get_attributes_by_entity_type_request_entity_type import GetAttributesByEntityTypeRequestEntityType
    from .update_attribute_request_attribute_type import UpdateAttributeRequestAttributeType
    from .update_attribute_request_entity_type import UpdateAttributeRequestEntityType
_dynamic_imports: typing.Dict[str, str] = {
    "CreateAttributeRequestAttributeType": ".create_attribute_request_attribute_type",
    "CreateAttributeRequestEntityType": ".create_attribute_request_entity_type",
    "CreateAttributeRequestUnit": ".create_attribute_request_unit",
    "DeleteAttributesRequestEntityType": ".delete_attributes_request_entity_type",
    "GetAttributeRequestEntityType": ".get_attribute_request_entity_type",
    "GetAttributesByEntityTypeRequestEntityType": ".get_attributes_by_entity_type_request_entity_type",
    "UpdateAttributeRequestAttributeType": ".update_attribute_request_attribute_type",
    "UpdateAttributeRequestEntityType": ".update_attribute_request_entity_type",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "CreateAttributeRequestAttributeType",
    "CreateAttributeRequestEntityType",
    "CreateAttributeRequestUnit",
    "DeleteAttributesRequestEntityType",
    "GetAttributeRequestEntityType",
    "GetAttributesByEntityTypeRequestEntityType",
    "UpdateAttributeRequestAttributeType",
    "UpdateAttributeRequestEntityType",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .dvir_defects_object_v_20220913_response_body import DvirDefectsObjectV20220913ResponseBody
from .goa_trailer_tiny_response_response_body import GoaTrailerTinyResponseResponseBody
from .workflow_author_signature_object_response_body import WorkflowAuthorSignatureObjectResponseBody
from .workflow_dvir_object_response_body_safety_status import WorkflowDvirObjectResponseBodySafetyStatus
from .workflow_dvir_object_response_body_type import WorkflowDvirObjectResponseBodyType


class WorkflowDvirObjectResponseBody(UniversalBaseModel):
    """
    A DVIR description
    """

    author_signature: typing_extensions.Annotated[
        WorkflowAuthorSignatureObjectResponseBody,
        FieldMetadata(alias="authorSignature"),
        pydantic.Field(alias="authorSignature"),
    ]
    defects: typing.Optional[typing.List[DvirDefectsObjectV20220913ResponseBody]] = pydantic.Field(default=None)
    """
    Defects registered for the DVIR.
    """

    end_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="endTime"),
        pydantic.Field(
            alias="endTime",
            description="Time when the driver signed and completed this DVIR. UTC timestamp in RFC 3339 format.",
        ),
    ]
    formatted_location: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="formattedLocation"),
        pydantic.Field(
            alias="formattedLocation",
            description="Optional string if your jurisdiction requires a location of the DVIR.",
        ),
    ] = None
    id: str = pydantic.Field()
    """
    The unique id of the DVIR
    """

    mechanic_notes: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="mechanicNotes"),
        pydantic.Field(alias="mechanicNotes", description="The mechanics notes on the DVIR."),
    ] = None
    needs_correction: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="needsCorrection"),
        pydantic.Field(alias="needsCorrection", description="Indicates if a defect needs correction."),
    ]
    odometer_meters: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="odometerMeters"),
        pydantic.Field(alias="odometerMeters", description="The odometer reading in meters."),
    ] = None
    safety_status: typing_extensions.Annotated[
        WorkflowDvirObjectResponseBodySafetyStatus,
        FieldMetadata(alias="safetyStatus"),
        pydantic.Field(
            alias="safetyStatus",
            description="The condition of vehicle on which DVIR was done.  Valid values: `safe`, `unsafe`, `resolved`",
        ),
    ]
    second_signature: typing_extensions.Annotated[
        typing.Optional[WorkflowAuthorSignatureObjectResponseBody],
        FieldMetadata(alias="secondSignature"),
        pydantic.Field(alias="secondSignature"),
    ] = None
    start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="startTime"),
        pydantic.Field(
            alias="startTime", description="Time when driver began filling out this DVIR in RFC 3339 format."
        ),
    ]
    third_signature: typing_extensions.Annotated[
        typing.Optional[WorkflowAuthorSignatureObjectResponseBody],
        FieldMetadata(alias="thirdSignature"),
        pydantic.Field(alias="thirdSignature"),
    ] = None
    trailer: typing.Optional[GoaTrailerTinyResponseResponseBody] = None
    type: WorkflowDvirObjectResponseBodyType = pydantic.Field()
    """
    Inspection type of the DVIR.  Valid values: `preTrip`, `postTrip`, `mechanic`, `unspecified`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .work_order_money_object_response_body_currency import WorkOrderMoneyObjectResponseBodyCurrency


class WorkOrderMoneyObjectResponseBody(UniversalBaseModel):
    """
    A specified amount of money.
    """

    amount: str = pydantic.Field()
    """
    Amount of the currency.
    """

    currency: WorkOrderMoneyObjectResponseBodyCurrency = pydantic.Field()
    """
    Type of the currency. We currently only support usd.  Valid values: `usd`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .asset_data_input import AssetDataInput


class AssetDataOutput(UniversalBaseModel):
    data_group: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="dataGroup"),
        pydantic.Field(alias="dataGroup", description="Name of the data group that the data output is associated with"),
    ] = None
    data_input: typing_extensions.Annotated[
        typing.Optional[AssetDataInput], FieldMetadata(alias="dataInput"), pydantic.Field(alias="dataInput")
    ] = None
    device_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="deviceId"),
        pydantic.Field(alias="deviceId", description="ID of the device that the data output is configured on"),
    ] = None
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the data output
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the data output
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .specific_vehicle_fault_code_object_request_body_type import SpecificVehicleFaultCodeObjectRequestBodyType


class SpecificVehicleFaultCodeObjectRequestBody(UniversalBaseModel):
    """
    A specific vehicle fault code.
    """

    fault_code: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="faultCode"),
        pydantic.Field(alias="faultCode", description="The specific fault code name."),
    ]
    type: SpecificVehicleFaultCodeObjectRequestBodyType = pydantic.Field()
    """
    The specific fault code type.  Valid values: `INVALID_FAULT_CODE_TYPE`, `J1939_DTC`, `J1939_SPN`, `PASSENGER_DTC`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .ambient_temperature_details_object_request_body_operation import (
    AmbientTemperatureDetailsObjectRequestBodyOperation,
)


class AmbientTemperatureDetailsObjectRequestBody(UniversalBaseModel):
    """
    Details specific to Ambient Temperature.
    """

    cargo_is_full: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="cargoIsFull"),
        pydantic.Field(alias="cargoIsFull", description="Whether the cargo is full."),
    ] = None
    doors_are_closed: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="doorsAreClosed"),
        pydantic.Field(alias="doorsAreClosed", description="Whether the doors are closed."),
    ] = None
    min_duration_milliseconds: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="minDurationMilliseconds"),
        pydantic.Field(
            alias="minDurationMilliseconds",
            description="The number of milliseconds the trigger needs to stay active before alerting.",
        ),
    ]
    operation: AmbientTemperatureDetailsObjectRequestBodyOperation = pydantic.Field()
    """
    How to evaluate the threshold.  Valid values: `GREATER`, `INSIDE_RANGE`, `LESS`, `OUTSIDE_RANGE`
    """

    temperature_celcius: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="temperatureCelcius"),
        pydantic.Field(alias="temperatureCelcius", description="The temperature in Celcius threshold value."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DefectTypesResponseDataResponseBodySeverity = typing.Union[
    typing.Literal["major", "minor", "notApplicable"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class GatewayAssetResponseObjectResponseBody(UniversalBaseModel):
    """
    An object containing information about the asset the gateway is installed on
    """

    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique Samsara ID of the asset where the gateway is installed. This is automatically generated when the asset is created and cannot be changed. Use this ID on PATCH vehicle, equipment, or trailer endpoints to update the asset
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .goa_driver_tiny_response_response_body import GoaDriverTinyResponseResponseBody
from .minimal_route_stop_response_body import MinimalRouteStopResponseBody
from .route_stop_details_object_response_body_operation import RouteStopDetailsObjectResponseBodyOperation
from .route_stop_details_object_response_body_type import RouteStopDetailsObjectResponseBodyType
from .vehicle_with_gateway_tiny_response_response_body import VehicleWithGatewayTinyResponseResponseBody
from .webhook_route_response_object_response_body import WebhookRouteResponseObjectResponseBody


class RouteStopDetailsObjectResponseBody(UniversalBaseModel):
    driver: typing.Optional[GoaDriverTinyResponseResponseBody] = None
    operation: typing.Optional[RouteStopDetailsObjectResponseBodyOperation] = pydantic.Field(default=None)
    """
    The operation that was performed as part of this route update.  Valid values: `stop arrived`, `stop departed`
    """

    route: WebhookRouteResponseObjectResponseBody
    route_stop_details: typing_extensions.Annotated[
        MinimalRouteStopResponseBody, FieldMetadata(alias="routeStopDetails"), pydantic.Field(alias="routeStopDetails")
    ]
    time: dt.datetime = pydantic.Field()
    """
    The timestamp of the route in RFC 3339 format.
    """

    type: RouteStopDetailsObjectResponseBodyType = pydantic.Field()
    """
    The type of route update. The route tracking updates occur as a route is completed and stops transition from one state to another. Currently only Route Tracking updates are supported, but this will change in the future when additional types are added.  Valid values: `route tracking`
    """

    vehicle: typing.Optional[VehicleWithGatewayTinyResponseResponseBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DeviceRecoveryRecoverAssetRequestBodyRecoveryStatus = typing.Union[typing.Literal["YES", "NO", "NOT_SURE"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

PatchSafetyEventsDismissalReasonBodyRequestBodyCode = typing.Union[
    typing.Literal["incorrect", "minorEvent", "other"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .current_duty_status_hos_status_type import CurrentDutyStatusHosStatusType


class CurrentDutyStatus(UniversalBaseModel):
    """
    The current HOS status type and time the driver started being in this status.
    """

    hos_status_type: typing_extensions.Annotated[
        typing.Optional[CurrentDutyStatusHosStatusType],
        FieldMetadata(alias="hosStatusType"),
        pydantic.Field(
            alias="hosStatusType",
            description="The Hours of Service status type.  If the driver app is disconnected, an empty string will be returned. To reconnect the app and return updated values, drivers should have the app open with good cell service. Valid values: `offDuty`, `sleeperBed`, `driving`, `onDuty`, `yardMove`, `personalConveyance`.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

QualificationRecordRequestFieldInputObjectRequestBodyType = typing.Union[
    typing.Literal["number", "text", "multiple_choice", "check_boxes", "datetime", "table", "media", "signature"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .connection_status_response_object_response_body_health_status import (
    ConnectionStatusResponseObjectResponseBodyHealthStatus,
)


class ConnectionStatusResponseObjectResponseBody(UniversalBaseModel):
    """
    An objecting containing information about the connectivity status of the gateway
    """

    health_status: typing_extensions.Annotated[
        typing.Optional[ConnectionStatusResponseObjectResponseBodyHealthStatus],
        FieldMetadata(alias="healthStatus"),
        pydantic.Field(
            alias="healthStatus",
            description="The most recent health status of the gateway.  Valid values: `Status Not Set`, `Connected`, `Not Installed`, `Power Source Off - Active Vehicle`, `Power Source Off - Inactive Vehicle`, `Weak Cellular Signal`, `Requires Investigation`, `Requires Charge`, `Unsupported Product`, `Low Battery. Replace Device.`, `Low Vehicle Battery`, `Unplugged`, `Low Charging State`, `Vehicle Off`, `Weak GPS Signal`, `Low Gateway Battery`, `Low Gateway Battery (AG24)`, `Low Gateway Battery (AG45)`, `Low Gateway Battery (AG26)`, `Low Gateway Battery (AG46)`, `Low Gateway Battery (AG46-P)`, `Temporarily Offline`, `Prolonged Offline`, `Recently Offline`, `Replacement Required`, `Status Unknown`",
        ),
    ] = None
    last_connected: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="lastConnected"),
        pydantic.Field(
            alias="lastConnected",
            description="The last time the gateway was connected in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import os
import typing

import httpx
from .core.api_error import ApiError
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .environment import SamsaraEnvironment

if typing.TYPE_CHECKING:
    from .addresses.client import AddressesClient, AsyncAddressesClient
    from .alerts.client import AlertsClient, AsyncAlertsClient
    from .assets.client import AssetsClient, AsyncAssetsClient
    from .attributes.client import AsyncAttributesClient, AttributesClient
    from .auth_token_for_driver.client import AsyncAuthTokenForDriverClient, AuthTokenForDriverClient
    from .beta_ap_is.client import AsyncBetaApIsClient, BetaApIsClient
    from .carrier_proposed_assignments.client import (
        AsyncCarrierProposedAssignmentsClient,
        CarrierProposedAssignmentsClient,
    )
    from .coaching.client import AsyncCoachingClient, CoachingClient
    from .contacts.client import AsyncContactsClient, ContactsClient
    from .documents.client import AsyncDocumentsClient, DocumentsClient
    from .driver_qr_codes.client import AsyncDriverQrCodesClient, DriverQrCodesClient
    from .driver_trailer_assignments.client import AsyncDriverTrailerAssignmentsClient, DriverTrailerAssignmentsClient
    from .driver_vehicle_assignments.client import AsyncDriverVehicleAssignmentsClient, DriverVehicleAssignmentsClient
    from .drivers.client import AsyncDriversClient, DriversClient
    from .equipment.client import AsyncEquipmentClient, EquipmentClient
    from .fleet.client import AsyncFleetClient, FleetClient
    from .forms.client import AsyncFormsClient, FormsClient
    from .fuel_and_energy.client import AsyncFuelAndEnergyClient, FuelAndEnergyClient
    from .gateways.client import AsyncGatewaysClient, GatewaysClient
    from .hours_of_service.client import AsyncHoursOfServiceClient, HoursOfServiceClient
    from .hubs.client import AsyncHubsClient, HubsClient
    from .idling.client import AsyncIdlingClient, IdlingClient
    from .ifta.client import AsyncIftaClient, IftaClient
    from .industrial.client import AsyncIndustrialClient, IndustrialClient
    from .issues.client import AsyncIssuesClient, IssuesClient
    from .legacy.client import AsyncLegacyClient, LegacyClient
    from .legacy_ap_is.client import AsyncLegacyApIsClient, LegacyApIsClient
    from .live_sharing_links.client import AsyncLiveSharingLinksClient, LiveSharingLinksClient
    from .location_and_speed.client import AsyncLocationAndSpeedClient, LocationAndSpeedClient
    from .maintenance.client import AsyncMaintenanceClient, MaintenanceClient
    from .media.client import AsyncMediaClient, MediaClient
    from .messages.client import AsyncMessagesClient, MessagesClient
    from .organization_info.client import AsyncOrganizationInfoClient, OrganizationInfoClient
    from .plans.client import AsyncPlansClient, PlansClient
    from .preview_ap_is.client import AsyncPreviewApIsClient, PreviewApIsClient
    from .route_events.client import AsyncRouteEventsClient, RouteEventsClient
    from .routes.client import AsyncRoutesClient, RoutesClient
    from .safety.client import AsyncSafetyClient, SafetyClient
    from .sensors.client import AsyncSensorsClient, SensorsClient
    from .settings.client import AsyncSettingsClient, SettingsClient
    from .speeding_intervals.client import AsyncSpeedingIntervalsClient, SpeedingIntervalsClient
    from .tachograph_eu_only.client import AsyncTachographEuOnlyClient, TachographEuOnlyClient
    from .tags.client import AsyncTagsClient, TagsClient
    from .trailer_assignments.client import AsyncTrailerAssignmentsClient, TrailerAssignmentsClient
    from .trailers.client import AsyncTrailersClient, TrailersClient
    from .training_assignments.client import AsyncTrainingAssignmentsClient, TrainingAssignmentsClient
    from .training_courses.client import AsyncTrainingCoursesClient, TrainingCoursesClient
    from .trips.client import AsyncTripsClient, TripsClient
    from .users.client import AsyncUsersClient, UsersClient
    from .vehicle_locations.client import AsyncVehicleLocationsClient, VehicleLocationsClient
    from .vehicle_stats.client import AsyncVehicleStatsClient, VehicleStatsClient
    from .vehicles.client import AsyncVehiclesClient, VehiclesClient
    from .webhooks.client import AsyncWebhooksClient, WebhooksClient
    from .work_orders.client import AsyncWorkOrdersClient, WorkOrdersClient


class Samsara:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : SamsaraEnvironment
        The environment to use for requests from the client. from .environment import SamsaraEnvironment



        Defaults to SamsaraEnvironment.PRODUCTION_API



    token : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from samsara import Samsara

    client = Samsara(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: SamsaraEnvironment = SamsaraEnvironment.PRODUCTION_API,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("SAMSARA_API_KEY"),
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        if token is None:
            raise ApiError(body="The client must be instantiated be either passing in token or setting SAMSARA_API_KEY")
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=token,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._addresses: typing.Optional[AddressesClient] = None
        self._alerts: typing.Optional[AlertsClient] = None
        self._assets: typing.Optional[AssetsClient] = None
        self._beta_ap_is: typing.Optional[BetaApIsClient] = None
        self._location_and_speed: typing.Optional[LocationAndSpeedClient] = None
        self._attributes: typing.Optional[AttributesClient] = None
        self._media: typing.Optional[MediaClient] = None
        self._coaching: typing.Optional[CoachingClient] = None
        self._contacts: typing.Optional[ContactsClient] = None
        self._maintenance: typing.Optional[MaintenanceClient] = None
        self._fuel_and_energy: typing.Optional[FuelAndEnergyClient] = None
        self._driver_trailer_assignments: typing.Optional[DriverTrailerAssignmentsClient] = None
        self._driver_qr_codes: typing.Optional[DriverQrCodesClient] = None
        self._carrier_proposed_assignments: typing.Optional[CarrierProposedAssignmentsClient] = None
        self._legacy_ap_is: typing.Optional[LegacyApIsClient] = None
        self._documents: typing.Optional[DocumentsClient] = None
        self._driver_vehicle_assignments: typing.Optional[DriverVehicleAssignmentsClient] = None
        self._drivers: typing.Optional[DriversClient] = None
        self._auth_token_for_driver: typing.Optional[AuthTokenForDriverClient] = None
        self._tachograph_eu_only: typing.Optional[TachographEuOnlyClient] = None
        self._equipment: typing.Optional[EquipmentClient] = None
        self._hours_of_service: typing.Optional[HoursOfServiceClient] = None
        self._ifta: typing.Optional[IftaClient] = None
        self._routes: typing.Optional[RoutesClient] = None
        self._settings: typing.Optional[SettingsClient] = None
        self._trailers: typing.Optional[TrailersClient] = None
        self._vehicles: typing.Optional[VehiclesClient] = None
        self._vehicle_locations: typing.Optional[VehicleLocationsClient] = None
        self._vehicle_stats: typing.Optional[VehicleStatsClient] = None
        self._forms: typing.Optional[FormsClient] = None
        self._gateways: typing.Optional[GatewaysClient] = None
        self._hubs: typing.Optional[HubsClient] = None
        self._plans: typing.Optional[PlansClient] = None
        self._idling: typing.Optional[IdlingClient] = None
        self._industrial: typing.Optional[IndustrialClient] = None
        self._issues: typing.Optional[IssuesClient] = None
        self._live_sharing_links: typing.Optional[LiveSharingLinksClient] = None
        self._work_orders: typing.Optional[WorkOrdersClient] = None
        self._organization_info: typing.Optional[OrganizationInfoClient] = None
        self._preview_ap_is: typing.Optional[PreviewApIsClient] = None
        self._route_events: typing.Optional[RouteEventsClient] = None
        self._safety: typing.Optional[SafetyClient] = None
        self._speeding_intervals: typing.Optional[SpeedingIntervalsClient] = None
        self._tags: typing.Optional[TagsClient] = None
        self._training_assignments: typing.Optional[TrainingAssignmentsClient] = None
        self._training_courses: typing.Optional[TrainingCoursesClient] = None
        self._trips: typing.Optional[TripsClient] = None
        self._users: typing.Optional[UsersClient] = None
        self._legacy: typing.Optional[LegacyClient] = None
        self._messages: typing.Optional[MessagesClient] = None
        self._trailer_assignments: typing.Optional[TrailerAssignmentsClient] = None
        self._sensors: typing.Optional[SensorsClient] = None
        self._webhooks: typing.Optional[WebhooksClient] = None
        self._fleet: typing.Optional[FleetClient] = None

    @property
    def addresses(self):
        if self._addresses is None:
            from .addresses.client import AddressesClient  # noqa: E402

            self._addresses = AddressesClient(client_wrapper=self._client_wrapper)
        return self._addresses

    @property
    def alerts(self):
        if self._alerts is None:
            from .alerts.client import AlertsClient  # noqa: E402

            self._alerts = AlertsClient(client_wrapper=self._client_wrapper)
        return self._alerts

    @property
    def assets(self):
        if self._assets is None:
            from .assets.client import AssetsClient  # noqa: E402

            self._assets = AssetsClient(client_wrapper=self._client_wrapper)
        return self._assets

    @property
    def beta_ap_is(self):
        if self._beta_ap_is is None:
            from .beta_ap_is.client import BetaApIsClient  # noqa: E402

            self._beta_ap_is = BetaApIsClient(client_wrapper=self._client_wrapper)
        return self._beta_ap_is

    @property
    def location_and_speed(self):
        if self._location_and_speed is None:
            from .location_and_speed.client import LocationAndSpeedClient  # noqa: E402

            self._location_and_speed = LocationAndSpeedClient(client_wrapper=self._client_wrapper)
        return self._location_and_speed

    @property
    def attributes(self):
        if self._attributes is None:
            from .attributes.client import AttributesClient  # noqa: E402

            self._attributes = AttributesClient(client_wrapper=self._client_wrapper)
        return self._attributes

    @property
    def media(self):
        if self._media is None:
            from .media.client import MediaClient  # noqa: E402

            self._media = MediaClient(client_wrapper=self._client_wrapper)
        return self._media

    @property
    def coaching(self):
        if self._coaching is None:
            from .coaching.client import CoachingClient  # noqa: E402

            self._coaching = CoachingClient(client_wrapper=self._client_wrapper)
        return self._coaching

    @property
    def contacts(self):
        if self._contacts is None:
            from .contacts.client import ContactsClient  # noqa: E402

            self._contacts = ContactsClient(client_wrapper=self._client_wrapper)
        return self._contacts

    @property
    def maintenance(self):
        if self._maintenance is None:
            from .maintenance.client import MaintenanceClient  # noqa: E402

            self._maintenance = MaintenanceClient(client_wrapper=self._client_wrapper)
        return self._maintenance

    @property
    def fuel_and_energy(self):
        if self._fuel_and_energy is None:
            from .fuel_and_energy.client import FuelAndEnergyClient  # noqa: E402

            self._fuel_and_energy = FuelAndEnergyClient(client_wrapper=self._client_wrapper)
        return self._fuel_and_energy

    @property
    def driver_trailer_assignments(self):
        if self._driver_trailer_assignments is None:
            from .driver_trailer_assignments.client import DriverTrailerAssignmentsClient  # noqa: E402

            self._driver_trailer_assignments = DriverTrailerAssignmentsClient(client_wrapper=self._client_wrapper)
        return self._driver_trailer_assignments

    @property
    def driver_qr_codes(self):
        if self._driver_qr_codes is None:
            from .driver_qr_codes.client import DriverQrCodesClient  # noqa: E402

            self._driver_qr_codes = DriverQrCodesClient(client_wrapper=self._client_wrapper)
        return self._driver_qr_codes

    @property
    def carrier_proposed_assignments(self):
        if self._carrier_proposed_assignments is None:
            from .carrier_proposed_assignments.client import CarrierProposedAssignmentsClient  # noqa: E402

            self._carrier_proposed_assignments = CarrierProposedAssignmentsClient(client_wrapper=self._client_wrapper)
        return self._carrier_proposed_assignments

    @property
    def legacy_ap_is(self):
        if self._legacy_ap_is is None:
            from .legacy_ap_is.client import LegacyApIsClient  # noqa: E402

            self._legacy_ap_is = LegacyApIsClient(client_wrapper=self._client_wrapper)
        return self._legacy_ap_is

    @property
    def documents(self):
        if self._documents is None:
            from .documents.client import DocumentsClient  # noqa: E402

            self._documents = DocumentsClient(client_wrapper=self._client_wrapper)
        return self._documents

    @property
    def driver_vehicle_assignments(self):
        if self._driver_vehicle_assignments is None:
            from .driver_vehicle_assignments.client import DriverVehicleAssignmentsClient  # noqa: E402

            self._driver_vehicle_assignments = DriverVehicleAssignmentsClient(client_wrapper=self._client_wrapper)
        return self._driver_vehicle_assignments

    @property
    def drivers(self):
        if self._drivers is None:
            from .drivers.client import DriversClient  # noqa: E402

            self._drivers = DriversClient(client_wrapper=self._client_wrapper)
        return self._drivers

    @property
    def auth_token_for_driver(self):
        if self._auth_token_for_driver is None:
            from .auth_token_for_driver.client import AuthTokenForDriverClient  # noqa: E402

            self._auth_token_for_driver = AuthTokenForDriverClient(client_wrapper=self._client_wrapper)
        return self._auth_token_for_driver

    @property
    def tachograph_eu_only(self):
        if self._tachograph_eu_only is None:
            from .tachograph_eu_only.client import TachographEuOnlyClient  # noqa: E402

            self._tachograph_eu_only = TachographEuOnlyClient(client_wrapper=self._client_wrapper)
        return self._tachograph_eu_only

    @property
    def equipment(self):
        if self._equipment is None:
            from .equipment.client import EquipmentClient  # noqa: E402

            self._equipment = EquipmentClient(client_wrapper=self._client_wrapper)
        return self._equipment

    @property
    def hours_of_service(self):
        if self._hours_of_service is None:
            from .hours_of_service.client import HoursOfServiceClient  # noqa: E402

            self._hours_of_service = HoursOfServiceClient(client_wrapper=self._client_wrapper)
        return self._hours_of_service

    @property
    def ifta(self):
        if self._ifta is None:
            from .ifta.client import IftaClient  # noqa: E402

            self._ifta = IftaClient(client_wrapper=self._client_wrapper)
        return self._ifta

    @property
    def routes(self):
        if self._routes is None:
            from .routes.client import RoutesClient  # noqa: E402

            self._routes = RoutesClient(client_wrapper=self._client_wrapper)
        return self._routes

    @property
    def settings(self):
        if self._settings is None:
            from .settings.client import SettingsClient  # noqa: E402

            self._settings = SettingsClient(client_wrapper=self._client_wrapper)
        return self._settings

    @property
    def trailers(self):
        if self._trailers is None:
            from .trailers.client import TrailersClient  # noqa: E402

            self._trailers = TrailersClient(client_wrapper=self._client_wrapper)
        return self._trailers

    @property
    def vehicles(self):
        if self._vehicles is None:
            from .vehicles.client import VehiclesClient  # noqa: E402

            self._vehicles = VehiclesClient(client_wrapper=self._client_wrapper)
        return self._vehicles

    @property
    def vehicle_locations(self):
        if self._vehicle_locations is None:
            from .vehicle_locations.client import VehicleLocationsClient  # noqa: E402

            self._vehicle_locations = VehicleLocationsClient(client_wrapper=self._client_wrapper)
        return self._vehicle_locations

    @property
    def vehicle_stats(self):
        if self._vehicle_stats is None:
            from .vehicle_stats.client import VehicleStatsClient  # noqa: E402

            self._vehicle_stats = VehicleStatsClient(client_wrapper=self._client_wrapper)
        return self._vehicle_stats

    @property
    def forms(self):
        if self._forms is None:
            from .forms.client import FormsClient  # noqa: E402

            self._forms = FormsClient(client_wrapper=self._client_wrapper)
        return self._forms

    @property
    def gateways(self):
        if self._gateways is None:
            from .gateways.client import GatewaysClient  # noqa: E402

            self._gateways = GatewaysClient(client_wrapper=self._client_wrapper)
        return self._gateways

    @property
    def hubs(self):
        if self._hubs is None:
            from .hubs.client import HubsClient  # noqa: E402

            self._hubs = HubsClient(client_wrapper=self._client_wrapper)
        return self._hubs

    @property
    def plans(self):
        if self._plans is None:
            from .plans.client import PlansClient  # noqa: E402

            self._plans = PlansClient(client_wrapper=self._client_wrapper)
        return self._plans

    @property
    def idling(self):
        if self._idling is None:
            from .idling.client import IdlingClient  # noqa: E402

            self._idling = IdlingClient(client_wrapper=self._client_wrapper)
        return self._idling

    @property
    def industrial(self):
        if self._industrial is None:
            from .industrial.client import IndustrialClient  # noqa: E402

            self._industrial = IndustrialClient(client_wrapper=self._client_wrapper)
        return self._industrial

    @property
    def issues(self):
        if self._issues is None:
            from .issues.client import IssuesClient  # noqa: E402

            self._issues = IssuesClient(client_wrapper=self._client_wrapper)
        return self._issues

    @property
    def live_sharing_links(self):
        if self._live_sharing_links is None:
            from .live_sharing_links.client import LiveSharingLinksClient  # noqa: E402

            self._live_sharing_links = LiveSharingLinksClient(client_wrapper=self._client_wrapper)
        return self._live_sharing_links

    @property
    def work_orders(self):
        if self._work_orders is None:
            from .work_orders.client import WorkOrdersClient  # noqa: E402

            self._work_orders = WorkOrdersClient(client_wrapper=self._client_wrapper)
        return self._work_orders

    @property
    def organization_info(self):
        if self._organization_info is None:
            from .organization_info.client import OrganizationInfoClient  # noqa: E402

            self._organization_info = OrganizationInfoClient(client_wrapper=self._client_wrapper)
        return self._organization_info

    @property
    def preview_ap_is(self):
        if self._preview_ap_is is None:
            from .preview_ap_is.client import PreviewApIsClient  # noqa: E402

            self._preview_ap_is = PreviewApIsClient(client_wrapper=self._client_wrapper)
        return self._preview_ap_is

    @property
    def route_events(self):
        if self._route_events is None:
            from .route_events.client import RouteEventsClient  # noqa: E402

            self._route_events = RouteEventsClient(client_wrapper=self._client_wrapper)
        return self._route_events

    @property
    def safety(self):
        if self._safety is None:
            from .safety.client import SafetyClient  # noqa: E402

            self._safety = SafetyClient(client_wrapper=self._client_wrapper)
        return self._safety

    @property
    def speeding_intervals(self):
        if self._speeding_intervals is None:
            from .speeding_intervals.client import SpeedingIntervalsClient  # noqa: E402

            self._speeding_intervals = SpeedingIntervalsClient(client_wrapper=self._client_wrapper)
        return self._speeding_intervals

    @property
    def tags(self):
        if self._tags is None:
            from .tags.client import TagsClient  # noqa: E402

            self._tags = TagsClient(client_wrapper=self._client_wrapper)
        return self._tags

    @property
    def training_assignments(self):
        if self._training_assignments is None:
            from .training_assignments.client import TrainingAssignmentsClient  # noqa: E402

            self._training_assignments = TrainingAssignmentsClient(client_wrapper=self._client_wrapper)
        return self._training_assignments

    @property
    def training_courses(self):
        if self._training_courses is None:
            from .training_courses.client import TrainingCoursesClient  # noqa: E402

            self._training_courses = TrainingCoursesClient(client_wrapper=self._client_wrapper)
        return self._training_courses

    @property
    def trips(self):
        if self._trips is None:
            from .trips.client import TripsClient  # noqa: E402

            self._trips = TripsClient(client_wrapper=self._client_wrapper)
        return self._trips

    @property
    def users(self):
        if self._users is None:
            from .users.client import UsersClient  # noqa: E402

            self._users = UsersClient(client_wrapper=self._client_wrapper)
        return self._users

    @property
    def legacy(self):
        if self._legacy is None:
            from .legacy.client import LegacyClient  # noqa: E402

            self._legacy = LegacyClient(client_wrapper=self._client_wrapper)
        return self._legacy

    @property
    def messages(self):
        if self._messages is None:
            from .messages.client import MessagesClient  # noqa: E402

            self._messages = MessagesClient(client_wrapper=self._client_wrapper)
        return self._messages

    @property
    def trailer_assignments(self):
        if self._trailer_assignments is None:
            from .trailer_assignments.client import TrailerAssignmentsClient  # noqa: E402

            self._trailer_assignments = TrailerAssignmentsClient(client_wrapper=self._client_wrapper)
        return self._trailer_assignments

    @property
    def sensors(self):
        if self._sensors is None:
            from .sensors.client import SensorsClient  # noqa: E402

            self._sensors = SensorsClient(client_wrapper=self._client_wrapper)
        return self._sensors

    @property
    def webhooks(self):
        if self._webhooks is None:
            from .webhooks.client import WebhooksClient  # noqa: E402

            self._webhooks = WebhooksClient(client_wrapper=self._client_wrapper)
        return self._webhooks

    @property
    def fleet(self):
        if self._fleet is None:
            from .fleet.client import FleetClient  # noqa: E402

            self._fleet = FleetClient(client_wrapper=self._client_wrapper)
        return self._fleet


class AsyncSamsara:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : SamsaraEnvironment
        The environment to use for requests from the client. from .environment import SamsaraEnvironment



        Defaults to SamsaraEnvironment.PRODUCTION_API



    token : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from samsara import AsyncSamsara

    client = AsyncSamsara(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: SamsaraEnvironment = SamsaraEnvironment.PRODUCTION_API,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("SAMSARA_API_KEY"),
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        if token is None:
            raise ApiError(body="The client must be instantiated be either passing in token or setting SAMSARA_API_KEY")
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=token,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._addresses: typing.Optional[AsyncAddressesClient] = None
        self._alerts: typing.Optional[AsyncAlertsClient] = None
        self._assets: typing.Optional[AsyncAssetsClient] = None
        self._beta_ap_is: typing.Optional[AsyncBetaApIsClient] = None
        self._location_and_speed: typing.Optional[AsyncLocationAndSpeedClient] = None
        self._attributes: typing.Optional[AsyncAttributesClient] = None
        self._media: typing.Optional[AsyncMediaClient] = None
        self._coaching: typing.Optional[AsyncCoachingClient] = None
        self._contacts: typing.Optional[AsyncContactsClient] = None
        self._maintenance: typing.Optional[AsyncMaintenanceClient] = None
        self._fuel_and_energy: typing.Optional[AsyncFuelAndEnergyClient] = None
        self._driver_trailer_assignments: typing.Optional[AsyncDriverTrailerAssignmentsClient] = None
        self._driver_qr_codes: typing.Optional[AsyncDriverQrCodesClient] = None
        self._carrier_proposed_assignments: typing.Optional[AsyncCarrierProposedAssignmentsClient] = None
        self._legacy_ap_is: typing.Optional[AsyncLegacyApIsClient] = None
        self._documents: typing.Optional[AsyncDocumentsClient] = None
        self._driver_vehicle_assignments: typing.Optional[AsyncDriverVehicleAssignmentsClient] = None
        self._drivers: typing.Optional[AsyncDriversClient] = None
        self._auth_token_for_driver: typing.Optional[AsyncAuthTokenForDriverClient] = None
        self._tachograph_eu_only: typing.Optional[AsyncTachographEuOnlyClient] = None
        self._equipment: typing.Optional[AsyncEquipmentClient] = None
        self._hours_of_service: typing.Optional[AsyncHoursOfServiceClient] = None
        self._ifta: typing.Optional[AsyncIftaClient] = None
        self._routes: typing.Optional[AsyncRoutesClient] = None
        self._settings: typing.Optional[AsyncSettingsClient] = None
        self._trailers: typing.Optional[AsyncTrailersClient] = None
        self._vehicles: typing.Optional[AsyncVehiclesClient] = None
        self._vehicle_locations: typing.Optional[AsyncVehicleLocationsClient] = None
        self._vehicle_stats: typing.Optional[AsyncVehicleStatsClient] = None
        self._forms: typing.Optional[AsyncFormsClient] = None
        self._gateways: typing.Optional[AsyncGatewaysClient] = None
        self._hubs: typing.Optional[AsyncHubsClient] = None
        self._plans: typing.Optional[AsyncPlansClient] = None
        self._idling: typing.Optional[AsyncIdlingClient] = None
        self._industrial: typing.Optional[AsyncIndustrialClient] = None
        self._issues: typing.Optional[AsyncIssuesClient] = None
        self._live_sharing_links: typing.Optional[AsyncLiveSharingLinksClient] = None
        self._work_orders: typing.Optional[AsyncWorkOrdersClient] = None
        self._organization_info: typing.Optional[AsyncOrganizationInfoClient] = None
        self._preview_ap_is: typing.Optional[AsyncPreviewApIsClient] = None
        self._route_events: typing.Optional[AsyncRouteEventsClient] = None
        self._safety: typing.Optional[AsyncSafetyClient] = None
        self._speeding_intervals: typing.Optional[AsyncSpeedingIntervalsClient] = None
        self._tags: typing.Optional[AsyncTagsClient] = None
        self._training_assignments: typing.Optional[AsyncTrainingAssignmentsClient] = None
        self._training_courses: typing.Optional[AsyncTrainingCoursesClient] = None
        self._trips: typing.Optional[AsyncTripsClient] = None
        self._users: typing.Optional[AsyncUsersClient] = None
        self._legacy: typing.Optional[AsyncLegacyClient] = None
        self._messages: typing.Optional[AsyncMessagesClient] = None
        self._trailer_assignments: typing.Optional[AsyncTrailerAssignmentsClient] = None
        self._sensors: typing.Optional[AsyncSensorsClient] = None
        self._webhooks: typing.Optional[AsyncWebhooksClient] = None
        self._fleet: typing.Optional[AsyncFleetClient] = None

    @property
    def addresses(self):
        if self._addresses is None:
            from .addresses.client import AsyncAddressesClient  # noqa: E402

            self._addresses = AsyncAddressesClient(client_wrapper=self._client_wrapper)
        return self._addresses

    @property
    def alerts(self):
        if self._alerts is None:
            from .alerts.client import AsyncAlertsClient  # noqa: E402

            self._alerts = AsyncAlertsClient(client_wrapper=self._client_wrapper)
        return self._alerts

    @property
    def assets(self):
        if self._assets is None:
            from .assets.client import AsyncAssetsClient  # noqa: E402

            self._assets = AsyncAssetsClient(client_wrapper=self._client_wrapper)
        return self._assets

    @property
    def beta_ap_is(self):
        if self._beta_ap_is is None:
            from .beta_ap_is.client import AsyncBetaApIsClient  # noqa: E402

            self._beta_ap_is = AsyncBetaApIsClient(client_wrapper=self._client_wrapper)
        return self._beta_ap_is

    @property
    def location_and_speed(self):
        if self._location_and_speed is None:
            from .location_and_speed.client import AsyncLocationAndSpeedClient  # noqa: E402

            self._location_and_speed = AsyncLocationAndSpeedClient(client_wrapper=self._client_wrapper)
        return self._location_and_speed

    @property
    def attributes(self):
        if self._attributes is None:
            from .attributes.client import AsyncAttributesClient  # noqa: E402

            self._attributes = AsyncAttributesClient(client_wrapper=self._client_wrapper)
        return self._attributes

    @property
    def media(self):
        if self._media is None:
            from .media.client import AsyncMediaClient  # noqa: E402

            self._media = AsyncMediaClient(client_wrapper=self._client_wrapper)
        return self._media

    @property
    def coaching(self):
        if self._coaching is None:
            from .coaching.client import AsyncCoachingClient  # noqa: E402

            self._coaching = AsyncCoachingClient(client_wrapper=self._client_wrapper)
        return self._coaching

    @property
    def contacts(self):
        if self._contacts is None:
            from .contacts.client import AsyncContactsClient  # noqa: E402

            self._contacts = AsyncContactsClient(client_wrapper=self._client_wrapper)
        return self._contacts

    @property
    def maintenance(self):
        if self._maintenance is None:
            from .maintenance.client import AsyncMaintenanceClient  # noqa: E402

            self._maintenance = AsyncMaintenanceClient(client_wrapper=self._client_wrapper)
        return self._maintenance

    @property
    def fuel_and_energy(self):
        if self._fuel_and_energy is None:
            from .fuel_and_energy.client import AsyncFuelAndEnergyClient  # noqa: E402

            self._fuel_and_energy = AsyncFuelAndEnergyClient(client_wrapper=self._client_wrapper)
        return self._fuel_and_energy

    @property
    def driver_trailer_assignments(self):
        if self._driver_trailer_assignments is None:
            from .driver_trailer_assignments.client import AsyncDriverTrailerAssignmentsClient  # noqa: E402

            self._driver_trailer_assignments = AsyncDriverTrailerAssignmentsClient(client_wrapper=self._client_wrapper)
        return self._driver_trailer_assignments

    @property
    def driver_qr_codes(self):
        if self._driver_qr_codes is None:
            from .driver_qr_codes.client import AsyncDriverQrCodesClient  # noqa: E402

            self._driver_qr_codes = AsyncDriverQrCodesClient(client_wrapper=self._client_wrapper)
        return self._driver_qr_codes

    @property
    def carrier_proposed_assignments(self):
        if self._carrier_proposed_assignments is None:
            from .carrier_proposed_assignments.client import AsyncCarrierProposedAssignmentsClient  # noqa: E402

            self._carrier_proposed_assignments = AsyncCarrierProposedAssignmentsClient(
                client_wrapper=self._client_wrapper
            )
        return self._carrier_proposed_assignments

    @property
    def legacy_ap_is(self):
        if self._legacy_ap_is is None:
            from .legacy_ap_is.client import AsyncLegacyApIsClient  # noqa: E402

            self._legacy_ap_is = AsyncLegacyApIsClient(client_wrapper=self._client_wrapper)
        return self._legacy_ap_is

    @property
    def documents(self):
        if self._documents is None:
            from .documents.client import AsyncDocumentsClient  # noqa: E402

            self._documents = AsyncDocumentsClient(client_wrapper=self._client_wrapper)
        return self._documents

    @property
    def driver_vehicle_assignments(self):
        if self._driver_vehicle_assignments is None:
            from .driver_vehicle_assignments.client import AsyncDriverVehicleAssignmentsClient  # noqa: E402

            self._driver_vehicle_assignments = AsyncDriverVehicleAssignmentsClient(client_wrapper=self._client_wrapper)
        return self._driver_vehicle_assignments

    @property
    def drivers(self):
        if self._drivers is None:
            from .drivers.client import AsyncDriversClient  # noqa: E402

            self._drivers = AsyncDriversClient(client_wrapper=self._client_wrapper)
        return self._drivers

    @property
    def auth_token_for_driver(self):
        if self._auth_token_for_driver is None:
            from .auth_token_for_driver.client import AsyncAuthTokenForDriverClient  # noqa: E402

            self._auth_token_for_driver = AsyncAuthTokenForDriverClient(client_wrapper=self._client_wrapper)
        return self._auth_token_for_driver

    @property
    def tachograph_eu_only(self):
        if self._tachograph_eu_only is None:
            from .tachograph_eu_only.client import AsyncTachographEuOnlyClient  # noqa: E402

            self._tachograph_eu_only = AsyncTachographEuOnlyClient(client_wrapper=self._client_wrapper)
        return self._tachograph_eu_only

    @property
    def equipment(self):
        if self._equipment is None:
            from .equipment.client import AsyncEquipmentClient  # noqa: E402

            self._equipment = AsyncEquipmentClient(client_wrapper=self._client_wrapper)
        return self._equipment

    @property
    def hours_of_service(self):
        if self._hours_of_service is None:
            from .hours_of_service.client import AsyncHoursOfServiceClient  # noqa: E402

            self._hours_of_service = AsyncHoursOfServiceClient(client_wrapper=self._client_wrapper)
        return self._hours_of_service

    @property
    def ifta(self):
        if self._ifta is None:
            from .ifta.client import AsyncIftaClient  # noqa: E402

            self._ifta = AsyncIftaClient(client_wrapper=self._client_wrapper)
        return self._ifta

    @property
    def routes(self):
        if self._routes is None:
            from .routes.client import AsyncRoutesClient  # noqa: E402

            self._routes = AsyncRoutesClient(client_wrapper=self._client_wrapper)
        return self._routes

    @property
    def settings(self):
        if self._settings is None:
            from .settings.client import AsyncSettingsClient  # noqa: E402

            self._settings = AsyncSettingsClient(client_wrapper=self._client_wrapper)
        return self._settings

    @property
    def trailers(self):
        if self._trailers is None:
            from .trailers.client import AsyncTrailersClient  # noqa: E402

            self._trailers = AsyncTrailersClient(client_wrapper=self._client_wrapper)
        return self._trailers

    @property
    def vehicles(self):
        if self._vehicles is None:
            from .vehicles.client import AsyncVehiclesClient  # noqa: E402

            self._vehicles = AsyncVehiclesClient(client_wrapper=self._client_wrapper)
        return self._vehicles

    @property
    def vehicle_locations(self):
        if self._vehicle_locations is None:
            from .vehicle_locations.client import AsyncVehicleLocationsClient  # noqa: E402

            self._vehicle_locations = AsyncVehicleLocationsClient(client_wrapper=self._client_wrapper)
        return self._vehicle_locations

    @property
    def vehicle_stats(self):
        if self._vehicle_stats is None:
            from .vehicle_stats.client import AsyncVehicleStatsClient  # noqa: E402

            self._vehicle_stats = AsyncVehicleStatsClient(client_wrapper=self._client_wrapper)
        return self._vehicle_stats

    @property
    def forms(self):
        if self._forms is None:
            from .forms.client import AsyncFormsClient  # noqa: E402

            self._forms = AsyncFormsClient(client_wrapper=self._client_wrapper)
        return self._forms

    @property
    def gateways(self):
        if self._gateways is None:
            from .gateways.client import AsyncGatewaysClient  # noqa: E402

            self._gateways = AsyncGatewaysClient(client_wrapper=self._client_wrapper)
        return self._gateways

    @property
    def hubs(self):
        if self._hubs is None:
            from .hubs.client import AsyncHubsClient  # noqa: E402

            self._hubs = AsyncHubsClient(client_wrapper=self._client_wrapper)
        return self._hubs

    @property
    def plans(self):
        if self._plans is None:
            from .plans.client import AsyncPlansClient  # noqa: E402

            self._plans = AsyncPlansClient(client_wrapper=self._client_wrapper)
        return self._plans

    @property
    def idling(self):
        if self._idling is None:
            from .idling.client import AsyncIdlingClient  # noqa: E402

            self._idling = AsyncIdlingClient(client_wrapper=self._client_wrapper)
        return self._idling

    @property
    def industrial(self):
        if self._industrial is None:
            from .industrial.client import AsyncIndustrialClient  # noqa: E402

            self._industrial = AsyncIndustrialClient(client_wrapper=self._client_wrapper)
        return self._industrial

    @property
    def issues(self):
        if self._issues is None:
            from .issues.client import AsyncIssuesClient  # noqa: E402

            self._issues = AsyncIssuesClient(client_wrapper=self._client_wrapper)
        return self._issues

    @property
    def live_sharing_links(self):
        if self._live_sharing_links is None:
            from .live_sharing_links.client import AsyncLiveSharingLinksClient  # noqa: E402

            self._live_sharing_links = AsyncLiveSharingLinksClient(client_wrapper=self._client_wrapper)
        return self._live_sharing_links

    @property
    def work_orders(self):
        if self._work_orders is None:
            from .work_orders.client import AsyncWorkOrdersClient  # noqa: E402

            self._work_orders = AsyncWorkOrdersClient(client_wrapper=self._client_wrapper)
        return self._work_orders

    @property
    def organization_info(self):
        if self._organization_info is None:
            from .organization_info.client import AsyncOrganizationInfoClient  # noqa: E402

            self._organization_info = AsyncOrganizationInfoClient(client_wrapper=self._client_wrapper)
        return self._organization_info

    @property
    def preview_ap_is(self):
        if self._preview_ap_is is None:
            from .preview_ap_is.client import AsyncPreviewApIsClient  # noqa: E402

            self._preview_ap_is = AsyncPreviewApIsClient(client_wrapper=self._client_wrapper)
        return self._preview_ap_is

    @property
    def route_events(self):
        if self._route_events is None:
            from .route_events.client import AsyncRouteEventsClient  # noqa: E402

            self._route_events = AsyncRouteEventsClient(client_wrapper=self._client_wrapper)
        return self._route_events

    @property
    def safety(self):
        if self._safety is None:
            from .safety.client import AsyncSafetyClient  # noqa: E402

            self._safety = AsyncSafetyClient(client_wrapper=self._client_wrapper)
        return self._safety

    @property
    def speeding_intervals(self):
        if self._speeding_intervals is None:
            from .speeding_intervals.client import AsyncSpeedingIntervalsClient  # noqa: E402

            self._speeding_intervals = AsyncSpeedingIntervalsClient(client_wrapper=self._client_wrapper)
        return self._speeding_intervals

    @property
    def tags(self):
        if self._tags is None:
            from .tags.client import AsyncTagsClient  # noqa: E402

            self._tags = AsyncTagsClient(client_wrapper=self._client_wrapper)
        return self._tags

    @property
    def training_assignments(self):
        if self._training_assignments is None:
            from .training_assignments.client import AsyncTrainingAssignmentsClient  # noqa: E402

            self._training_assignments = AsyncTrainingAssignmentsClient(client_wrapper=self._client_wrapper)
        return self._training_assignments

    @property
    def training_courses(self):
        if self._training_courses is None:
            from .training_courses.client import AsyncTrainingCoursesClient  # noqa: E402

            self._training_courses = AsyncTrainingCoursesClient(client_wrapper=self._client_wrapper)
        return self._training_courses

    @property
    def trips(self):
        if self._trips is None:
            from .trips.client import AsyncTripsClient  # noqa: E402

            self._trips = AsyncTripsClient(client_wrapper=self._client_wrapper)
        return self._trips

    @property
    def users(self):
        if self._users is None:
            from .users.client import AsyncUsersClient  # noqa: E402

            self._users = AsyncUsersClient(client_wrapper=self._client_wrapper)
        return self._users

    @property
    def legacy(self):
        if self._legacy is None:
            from .legacy.client import AsyncLegacyClient  # noqa: E402

            self._legacy = AsyncLegacyClient(client_wrapper=self._client_wrapper)
        return self._legacy

    @property
    def messages(self):
        if self._messages is None:
            from .messages.client import AsyncMessagesClient  # noqa: E402

            self._messages = AsyncMessagesClient(client_wrapper=self._client_wrapper)
        return self._messages

    @property
    def trailer_assignments(self):
        if self._trailer_assignments is None:
            from .trailer_assignments.client import AsyncTrailerAssignmentsClient  # noqa: E402

            self._trailer_assignments = AsyncTrailerAssignmentsClient(client_wrapper=self._client_wrapper)
        return self._trailer_assignments

    @property
    def sensors(self):
        if self._sensors is None:
            from .sensors.client import AsyncSensorsClient  # noqa: E402

            self._sensors = AsyncSensorsClient(client_wrapper=self._client_wrapper)
        return self._sensors

    @property
    def webhooks(self):
        if self._webhooks is None:
            from .webhooks.client import AsyncWebhooksClient  # noqa: E402

            self._webhooks = AsyncWebhooksClient(client_wrapper=self._client_wrapper)
        return self._webhooks

    @property
    def fleet(self):
        if self._fleet is None:
            from .fleet.client import AsyncFleetClient  # noqa: E402

            self._fleet = AsyncFleetClient(client_wrapper=self._client_wrapper)
        return self._fleet


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: SamsaraEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PlanObjectResponseBody(UniversalBaseModel):
    """
    Plan object
    """

    created_at: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAt"),
        pydantic.Field(alias="createdAt", description="The timestamp (in UTC) when the plan was created"),
    ]
    hub_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="hubId"),
        pydantic.Field(alias="hubId", description="The ID of the hub this plan belongs to"),
    ]
    id: str = pydantic.Field()
    """
    The Samsara-generated unique identifier (UUID) for the plan
    """

    name: str = pydantic.Field()
    """
    The name of the plan
    """

    shift_start_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="shiftStartTime"),
        pydantic.Field(alias="shiftStartTime", description="The shift start time for the plan in RFC 3339 format"),
    ]
    updated_at: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAt"),
        pydantic.Field(alias="updatedAt", description="The timestamp (in UTC) when the plan was last updated"),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class FunctionEffectsResponseBody(UniversalBaseModel):
    """
    Computed effects of the Function configuration.
    """

    next_scheduled_at_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="nextScheduledAtTime"),
        pydantic.Field(
            alias="nextScheduledAtTime",
            description="RFC 3339 timestamp of the next scheduled run. Omitted if no schedule is set.",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

FormsFieldDefinitionObjectResponseBodyAllowedAssetTypesItem = typing.Union[
    typing.Literal["vehicle", "trailer", "equipment", "unpoweredAsset"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_asset_reefer_response_reefer_stats_alarms_1 import V1AssetReeferResponseReeferStatsAlarms1
from .v_1_asset_reefer_response_reefer_stats_engine_hours import V1AssetReeferResponseReeferStatsEngineHours
from .v_1_asset_reefer_response_reefer_stats_fuel_percentage import V1AssetReeferResponseReeferStatsFuelPercentage
from .v_1_asset_reefer_response_reefer_stats_return_air_temp import V1AssetReeferResponseReeferStatsReturnAirTemp
from .v_1_asset_reefer_response_reefer_stats_set_point import V1AssetReeferResponseReeferStatsSetPoint
from .v_1_assets_reefer_reefer_stats_ambient_air_temperature import V1AssetsReeferReeferStatsAmbientAirTemperature
from .v_1_assets_reefer_reefer_stats_discharge_air_temperature import V1AssetsReeferReeferStatsDischargeAirTemperature
from .v_1_assets_reefer_reefer_stats_power_status import V1AssetsReeferReeferStatsPowerStatus


class V1AssetsReeferReeferStats(UniversalBaseModel):
    """
    Contains all the state changes of the reefer for the included stat types. Each state change is recorded independently, so the number of records in each array may differ depending on when that stat changed state. Stat types with a continuous value (such as temperature) will be recorded at different rates depending on the reefer, but generally readings have a frequency on the order of seconds.
    """

    ambient_air_temperature: typing_extensions.Annotated[
        typing.Optional[typing.List[V1AssetsReeferReeferStatsAmbientAirTemperature]],
        FieldMetadata(alias="ambientAirTemperature"),
        pydantic.Field(
            alias="ambientAirTemperature",
            description="Ambient temperature of the reefer. This is the temperature of the air around the Samsara Asset Gateway.",
        ),
    ] = None
    discharge_air_temperature: typing_extensions.Annotated[
        typing.Optional[typing.List[V1AssetsReeferReeferStatsDischargeAirTemperature]],
        FieldMetadata(alias="dischargeAirTemperature"),
        pydantic.Field(
            alias="dischargeAirTemperature",
            description="Discharge air temperature of the reefer. This is the temperature of the air as it leaves the cooling unit.",
        ),
    ] = None
    engine_hours: typing_extensions.Annotated[
        typing.Optional[typing.List[V1AssetReeferResponseReeferStatsEngineHours]],
        FieldMetadata(alias="engineHours"),
        pydantic.Field(alias="engineHours", description="Engine hours of the reefer"),
    ] = None
    fuel_percentage: typing_extensions.Annotated[
        typing.Optional[typing.List[V1AssetReeferResponseReeferStatsFuelPercentage]],
        FieldMetadata(alias="fuelPercentage"),
        pydantic.Field(alias="fuelPercentage", description="Fuel percentage of the reefer"),
    ] = None
    power_status: typing_extensions.Annotated[
        typing.Optional[typing.List[V1AssetsReeferReeferStatsPowerStatus]],
        FieldMetadata(alias="powerStatus"),
        pydantic.Field(alias="powerStatus", description="Power status of the reefer"),
    ] = None
    reefer_alarms: typing_extensions.Annotated[
        typing.Optional[typing.List[V1AssetReeferResponseReeferStatsAlarms1]],
        FieldMetadata(alias="reeferAlarms"),
        pydantic.Field(alias="reeferAlarms", description="Reefer alarms"),
    ] = None
    return_air_temperature: typing_extensions.Annotated[
        typing.Optional[typing.List[V1AssetReeferResponseReeferStatsReturnAirTemp]],
        FieldMetadata(alias="returnAirTemperature"),
        pydantic.Field(
            alias="returnAirTemperature",
            description="Return air temperature of the reefer. This is the temperature read by the reefer module (Carrier, Thermo King) that shows the temperature of the air as it enters the system.",
        ),
    ] = None
    set_point: typing_extensions.Annotated[
        typing.Optional[typing.List[V1AssetReeferResponseReeferStatsSetPoint]],
        FieldMetadata(alias="setPoint"),
        pydantic.Field(alias="setPoint", description="Set point temperature of the reefer"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .speeding_severity_level_response_body_severity_level import SpeedingSeverityLevelResponseBodySeverityLevel


class SpeedingSeverityLevelResponseBody(UniversalBaseModel):
    """
    The settings for a specific speeding severity level.
    """

    duration_ms: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="durationMs"),
        pydantic.Field(
            alias="durationMs",
            description="The amount of time the vehicle is speeding in this category before being attributed to this level",
        ),
    ]
    is_enabled: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isEnabled"),
        pydantic.Field(alias="isEnabled", description="Indicates the severity level is enabled"),
    ]
    severity_level: typing_extensions.Annotated[
        SpeedingSeverityLevelResponseBodySeverityLevel,
        FieldMetadata(alias="severityLevel"),
        pydantic.Field(
            alias="severityLevel",
            description="The severity level name.  Valid values: `light`, `moderate`, `heavy`, `severe`",
        ),
    ]
    speed_over_limit_threshold: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="speedOverLimitThreshold"),
        pydantic.Field(
            alias="speedOverLimitThreshold",
            description="The minimum speed above the speed limit that will get attributed to this severity level.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

RouteStopDetailsObjectResponseBodyOperation = typing.Union[typing.Literal["stop arrived", "stop departed"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class HubCapacityObjectResponseBody(UniversalBaseModel):
    """
    Hub capacity object
    """

    created_at: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAt"),
        pydantic.Field(alias="createdAt", description="Creation timestamp"),
    ]
    hub_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="hubId"), pydantic.Field(alias="hubId", description="Hub identifier")
    ]
    id: str = pydantic.Field()
    """
    Capacity identifier
    """

    name: str = pydantic.Field()
    """
    Capacity name
    """

    unit: str = pydantic.Field()
    """
    Unit of measurement
    """

    updated_at: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAt"),
        pydantic.Field(alias="updatedAt", description="Last update timestamp"),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class VehicleStatsFaultCodesVendorSpecificFields(UniversalBaseModel):
    """
    Vendor specific data for J1939 vehicles.
    """

    dtc_description: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="dtcDescription"),
        pydantic.Field(alias="dtcDescription", description="The DTC description, if available."),
    ] = None
    repair_instructions_url: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="repairInstructionsUrl"),
        pydantic.Field(
            alias="repairInstructionsUrl", description="A link to vendor repair instructions, if available."
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .work_order_money_object_response_body import WorkOrderMoneyObjectResponseBody


class WorkOrderUnallocatedLaborObjectResponseBody(UniversalBaseModel):
    """
    Unallocated labor from time entries not associated with any service task.
    """

    cost: WorkOrderMoneyObjectResponseBody
    time_minutes: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="timeMinutes"),
        pydantic.Field(alias="timeMinutes", description="The total unallocated labor time in minutes."),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .driver_efficiency import DriverEfficiency


class DriverEfficienciesResponseData(UniversalBaseModel):
    driver_summaries: typing_extensions.Annotated[
        typing.Optional[typing.List[DriverEfficiency]],
        FieldMetadata(alias="driverSummaries"),
        pydantic.Field(alias="driverSummaries", description="A list of driver and associated vehicle efficiency data."),
    ] = None
    summary_end_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="summaryEndTime"),
        pydantic.Field(
            alias="summaryEndTime",
            description="End time of the window for which this efficiency report was computed. Will be a UTC timestamp in RFC 3339 format. For example: `2020-03-16T16:00:00Z`",
        ),
    ] = None
    summary_start_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="summaryStartTime"),
        pydantic.Field(
            alias="summaryStartTime",
            description="Start time of the window for which this efficiency report was computed. Will be a UTC timestamp in RFC 3339 format. For example: `2020-03-15T16:00:00Z`",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .order_custom_property_input_request_body import OrderCustomPropertyInputRequestBody
from .order_quantity_input_request_body import OrderQuantityInputRequestBody
from .order_task_request_body import OrderTaskRequestBody


class OrderInputObjectRequestBody(UniversalBaseModel):
    """
    Input object for creating orders
    """

    custom_properties: typing_extensions.Annotated[
        typing.Optional[typing.List[OrderCustomPropertyInputRequestBody]],
        FieldMetadata(alias="customProperties"),
        pydantic.Field(alias="customProperties", description="An array of custom property values for the order"),
    ] = None
    customer_order_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="customerOrderId"),
        pydantic.Field(alias="customerOrderId", description="The customer-provided identifier for the order"),
    ]
    delivery: typing.Optional[OrderTaskRequestBody] = None
    hub_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="hubId"),
        pydantic.Field(alias="hubId", description="The ID of the hub the order belongs to"),
    ]
    pickup: typing.Optional[OrderTaskRequestBody] = None
    plan_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="planId"),
        pydantic.Field(alias="planId", description="The ID of the plan the order belongs to"),
    ]
    priority: typing.Optional[int] = pydantic.Field(default=None)
    """
    Priority of the order (e.g., 1 for high, 5 for low)
    """

    quantities: typing.Optional[typing.List[OrderQuantityInputRequestBody]] = pydantic.Field(default=None)
    """
    An array of quantities for the order
    """

    skills_required: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="skillsRequired"),
        pydantic.Field(alias="skillsRequired", description="An array of skill IDs required to fulfill the order"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

AddressAddressTypesItem = typing.Union[
    typing.Literal[
        "yard",
        "shortHaul",
        "workforceSite",
        "riskZone",
        "industrialSite",
        "alertsOnly",
        "agricultureSource",
        "avoidanceZone",
        "knownGPSJammingZone",
        "authorizedZone",
        "unauthorizedZone",
        "vendor",
        "inventory",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class V1VisionRunsByCameraResponseItemReportMetadata(UniversalBaseModel):
    items_per_minute: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="itemsPerMinute"), pydantic.Field(alias="itemsPerMinute")
    ] = None
    no_read_count: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="noReadCount"), pydantic.Field(alias="noReadCount")
    ] = None
    reject_count: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="rejectCount"), pydantic.Field(alias="rejectCount")
    ] = None
    success_count: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="successCount"), pydantic.Field(alias="successCount")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

TachographActivityState = typing.Union[
    typing.Literal["BREAK/REST", "WORK", "AVAILABILITY", "DRIVING", "UNKNOWN"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .distance_traveled_object_response_body import DistanceTraveledObjectResponseBody
from .driver_with_timezone_eld_settings_object_response_body import DriverWithTimezoneEldSettingsObjectResponseBody
from .duty_status_duration_object_response_body import DutyStatusDurationObjectResponseBody
from .log_meta_data_object_response_body import LogMetaDataObjectResponseBody
from .pending_duty_status_durations_object_response_body import PendingDutyStatusDurationsObjectResponseBody


class HosDailyLogsObjectResponseBody(UniversalBaseModel):
    distance_traveled: typing_extensions.Annotated[
        typing.Optional[DistanceTraveledObjectResponseBody],
        FieldMetadata(alias="distanceTraveled"),
        pydantic.Field(alias="distanceTraveled"),
    ] = None
    driver: DriverWithTimezoneEldSettingsObjectResponseBody
    duty_status_durations: typing_extensions.Annotated[
        typing.Optional[DutyStatusDurationObjectResponseBody],
        FieldMetadata(alias="dutyStatusDurations"),
        pydantic.Field(alias="dutyStatusDurations"),
    ] = None
    end_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="endTime"),
        pydantic.Field(
            alias="endTime",
            description="The end time of the daily log in RFC 3339 format. This will be calculated using timezone of the driver.",
        ),
    ]
    log_meta_data: typing_extensions.Annotated[
        typing.Optional[LogMetaDataObjectResponseBody],
        FieldMetadata(alias="logMetaData"),
        pydantic.Field(alias="logMetaData"),
    ] = None
    pending_duty_status_durations: typing_extensions.Annotated[
        typing.Optional[PendingDutyStatusDurationsObjectResponseBody],
        FieldMetadata(alias="pendingDutyStatusDurations"),
        pydantic.Field(alias="pendingDutyStatusDurations"),
    ] = None
    start_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="startTime"),
        pydantic.Field(
            alias="startTime",
            description="The start time of the daily log in RFC 3339 format. This will be calculated using timezone of the driver.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .routes_single_use_address_object_request_body import RoutesSingleUseAddressObjectRequestBody


class CreateRoutesStopRequestObjectRequestBody(UniversalBaseModel):
    address_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="addressId"),
        pydantic.Field(
            alias="addressId",
            description="ID of the address. An address [externalId](https://developers.samsara.com/docs/external-ids#using-external-ids) can also be used interchangeably here.",
        ),
    ] = None
    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the stop
    """

    notes: typing.Optional[str] = pydantic.Field(default=None)
    """
    Notes for the stop
    """

    ontime_window_after_arrival_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="ontimeWindowAfterArrivalMs"),
        pydantic.Field(
            alias="ontimeWindowAfterArrivalMs",
            description="Specifies the time window (in milliseconds) after a stop's scheduled arrival time during which the stop is considered 'on-time'.",
        ),
    ] = None
    ontime_window_before_arrival_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="ontimeWindowBeforeArrivalMs"),
        pydantic.Field(
            alias="ontimeWindowBeforeArrivalMs",
            description="Specifies the time window (in milliseconds) before a stop's scheduled arrival time during which the stop is considered 'on-time'.",
        ),
    ] = None
    scheduled_arrival_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="scheduledArrivalTime"),
        pydantic.Field(
            alias="scheduledArrivalTime",
            description="This is a required field for all stops EXCEPT the start and end, based on route start and stop settings selected.",
        ),
    ] = None
    scheduled_departure_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="scheduledDepartureTime"),
        pydantic.Field(
            alias="scheduledDepartureTime",
            description="This is a required field for all stops EXCEPT the start and end, based on route start and stop settings selected.",
        ),
    ] = None
    sequence_number: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="sequenceNumber"),
        pydantic.Field(
            alias="sequenceNumber",
            description="Manual sequence position for this stop. Only meaningful when route.settings.sequencingMethod=manual. Must be unique and positive when specified.",
        ),
    ] = None
    single_use_location: typing_extensions.Annotated[
        typing.Optional[RoutesSingleUseAddressObjectRequestBody],
        FieldMetadata(alias="singleUseLocation"),
        pydantic.Field(alias="singleUseLocation"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .forms_select_option_object_response_body import FormsSelectOptionObjectResponseBody
from .forms_table_field_definition_object_response_body_allowed_date_time_value_type import (
    FormsTableFieldDefinitionObjectResponseBodyAllowedDateTimeValueType,
)
from .forms_table_field_definition_object_response_body_type import FormsTableFieldDefinitionObjectResponseBodyType


class FormsTableFieldDefinitionObjectResponseBody(UniversalBaseModel):
    """
    Forms table field definition object.
    """

    allow_manual_entry: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="allowManualEntry"),
        pydantic.Field(
            alias="allowManualEntry",
            description="Indicates whether the field allows manual entry of a person. Only present for person fields.",
        ),
    ] = None
    allowed_date_time_value_type: typing_extensions.Annotated[
        typing.Optional[FormsTableFieldDefinitionObjectResponseBodyAllowedDateTimeValueType],
        FieldMetadata(alias="allowedDateTimeValueType"),
        pydantic.Field(
            alias="allowedDateTimeValueType",
            description="Type of date/time entry allowed for this question. Only present for datetime fields.  Valid values: `datetime`, `date`, `time`",
        ),
    ] = None
    filter_by_role_ids: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="filterByRoleIds"),
        pydantic.Field(
            alias="filterByRoleIds",
            description="List of role IDs to filter org users by, representing which roles are selectable people for this field. Only present for person fields.",
        ),
    ] = None
    id: str = pydantic.Field()
    """
    Identifier of the field.
    """

    include_drivers: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="includeDrivers"),
        pydantic.Field(
            alias="includeDrivers",
            description="Indicates whether the field includes drivers as selectable people. Only present for person fields.",
        ),
    ] = None
    include_users: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="includeUsers"),
        pydantic.Field(
            alias="includeUsers",
            description="Indicates whether the field includes users as selectable people. Only present for person fields.",
        ),
    ] = None
    label: str = pydantic.Field()
    """
    Label of the field.
    """

    num_decimal_places: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="numDecimalPlaces"),
        pydantic.Field(
            alias="numDecimalPlaces", description="Number of decimal places allowed. Only present for number fields."
        ),
    ] = None
    options: typing.Optional[typing.List[FormsSelectOptionObjectResponseBody]] = pydantic.Field(default=None)
    """
    List of select options for check boxes or multiple choice fields.
    """

    type: FormsTableFieldDefinitionObjectResponseBodyType = pydantic.Field()
    """
    Type of the field.  Valid values: `number`, `text`, `multiple_choice`, `check_boxes`, `media`, `datetime`, `signature`, `person`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_vision_step_results_item_caliper_angle_range import V1VisionStepResultsItemCaliperAngleRange
from .v_1_vision_step_results_item_caliper_contrast_range import V1VisionStepResultsItemCaliperContrastRange
from .v_1_vision_step_results_item_caliper_filter_polarity import V1VisionStepResultsItemCaliperFilterPolarity
from .v_1_vision_step_results_item_caliper_sharpness_range import V1VisionStepResultsItemCaliperSharpnessRange
from .v_1_vision_step_results_item_caliper_straightness_range import V1VisionStepResultsItemCaliperStraightnessRange


class V1VisionStepResultsItemCaliper(UniversalBaseModel):
    angle_range: typing_extensions.Annotated[
        typing.Optional[V1VisionStepResultsItemCaliperAngleRange],
        FieldMetadata(alias="angleRange"),
        pydantic.Field(alias="angleRange", description="The configured angle allowance range"),
    ] = None
    contrast_range: typing_extensions.Annotated[
        typing.Optional[V1VisionStepResultsItemCaliperContrastRange],
        FieldMetadata(alias="contrastRange"),
        pydantic.Field(alias="contrastRange", description="The configured contrast allowance range"),
    ] = None
    distance_found: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="distanceFound"),
        pydantic.Field(alias="distanceFound", description="The distance found between the found edges"),
    ] = None
    filter_polarity: typing_extensions.Annotated[
        typing.Optional[V1VisionStepResultsItemCaliperFilterPolarity],
        FieldMetadata(alias="filterPolarity"),
        pydantic.Field(
            alias="filterPolarity",
            description="The configured polarity for finding edges. Valid values: `LIGHT TO DARK`, `DARK TO LIGHT`.",
        ),
    ] = None
    max_distance: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="maxDistance"),
        pydantic.Field(alias="maxDistance", description="The maximum allowed distance threshold"),
    ] = None
    min_distance: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="minDistance"),
        pydantic.Field(alias="minDistance", description="The minumum allowed distance threshold"),
    ] = None
    sharpness_range: typing_extensions.Annotated[
        typing.Optional[V1VisionStepResultsItemCaliperSharpnessRange],
        FieldMetadata(alias="sharpnessRange"),
        pydantic.Field(alias="sharpnessRange", description="The configured sharpness allowance range"),
    ] = None
    straightness_range: typing_extensions.Annotated[
        typing.Optional[V1VisionStepResultsItemCaliperStraightnessRange],
        FieldMetadata(alias="straightnessRange"),
        pydantic.Field(alias="straightnessRange", description="The configured straightness allowance range"),
    ] = None
    unit: typing.Optional[str] = pydantic.Field(default=None)
    """
    The measurement unit of the distance found and the min and max distance threshold
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawFleetClient, RawFleetClient

if typing.TYPE_CHECKING:
    from .attributes.client import AsyncAttributesClient, AttributesClient
    from .carrier_proposed_assignments.client import (
        AsyncCarrierProposedAssignmentsClient,
        CarrierProposedAssignmentsClient,
    )


class FleetClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawFleetClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._carrier_proposed_assignments: typing.Optional[CarrierProposedAssignmentsClient] = None
        self._attributes: typing.Optional[AttributesClient] = None

    @property
    def with_raw_response(self) -> RawFleetClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawFleetClient
        """
        return self._raw_client

    @property
    def carrier_proposed_assignments(self):
        if self._carrier_proposed_assignments is None:
            from .carrier_proposed_assignments.client import CarrierProposedAssignmentsClient  # noqa: E402

            self._carrier_proposed_assignments = CarrierProposedAssignmentsClient(client_wrapper=self._client_wrapper)
        return self._carrier_proposed_assignments

    @property
    def attributes(self):
        if self._attributes is None:
            from .attributes.client import AttributesClient  # noqa: E402

            self._attributes = AttributesClient(client_wrapper=self._client_wrapper)
        return self._attributes


class AsyncFleetClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawFleetClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._carrier_proposed_assignments: typing.Optional[AsyncCarrierProposedAssignmentsClient] = None
        self._attributes: typing.Optional[AsyncAttributesClient] = None

    @property
    def with_raw_response(self) -> AsyncRawFleetClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawFleetClient
        """
        return self._raw_client

    @property
    def carrier_proposed_assignments(self):
        if self._carrier_proposed_assignments is None:
            from .carrier_proposed_assignments.client import AsyncCarrierProposedAssignmentsClient  # noqa: E402

            self._carrier_proposed_assignments = AsyncCarrierProposedAssignmentsClient(
                client_wrapper=self._client_wrapper
            )
        return self._carrier_proposed_assignments

    @property
    def attributes(self):
        if self._attributes is None:
            from .attributes.client import AsyncAttributesClient  # noqa: E402

            self._attributes = AsyncAttributesClient(client_wrapper=self._client_wrapper)
        return self._attributes
